/*
 * Copyright 2018 Scytl Secure Electronic Voting SA
 *
 * All rights reserved
 *
 * See our extended copyright notice in *file 'Copyright.txt' which is part of this source code package
 */
package com.scytl.products.ov.ag.ui.ws.rs.operation.admin;

import mockit.Deencapsulation;
import org.glassfish.hk2.utilities.binding.AbstractBinder;
import org.glassfish.jersey.server.ResourceConfig;
import org.glassfish.jersey.test.JerseyTest;
import org.glassfish.jersey.test.TestProperties;
import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.contrib.java.lang.system.RestoreSystemProperties;
import org.junit.rules.TestRule;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.slf4j.Logger;

import okhttp3.ResponseBody;
import retrofit2.Call;
import retrofit2.Response;

import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.client.Entity;
import javax.ws.rs.core.Application;
import javax.ws.rs.core.MediaType;

import java.io.IOException;
import java.util.ArrayList;

import com.scytl.products.ov.ag.services.infrastructure.remote.admin.ElectionInformationAdminClient;
import com.scytl.products.ov.commons.infrastructure.remote.client.RestClientInterceptor;
import com.scytl.products.ov.commons.tracking.TrackIdGenerator;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.when;

public class ElectoralDataEIResourceTest extends JerseyTest {

    private static final String ADMIN_BOARD_ID = "1a";

    private static final String ELECTION_EVENT_ID = "1e";

    private static final String ELECTORAL_AUTHORITY_ID = "1ea";
    
    private static final String TRACK_ID = "trackId";

    private static final String TENANT_ID = "100t";

    private static final String X_FORWARDER_VALUE = ",";

    private static final String SIGNATURE = "signature";

    private static final String ORIGINATOR = "originator";

    private static final String URL_SAVE_EE_CONTEXT_DATA = ElectoralDataEIResource.RESOURCE_PATH + "/" + ElectoralDataEIResource.SAVE_ELECTORAL_DATA;
    
    @Rule
    public TestRule restoreSystemProperties = new RestoreSystemProperties();

    @Mock
    Logger logger;
    
    @Mock
    TrackIdGenerator trackIdGenerator;

    @Mock
    HttpServletRequest servletRequest;

    @Mock
    ElectionInformationAdminClient electionInformationAdminClient;

    ElectoralDataEIResource sut;

    @Test
    public void saveElectionEventData() throws IOException {

        int mockedInvocationStatus = 200;
        
        @SuppressWarnings("unchecked")
		Call<ResponseBody> callMock = (Call<ResponseBody>) Mockito.mock(Call.class);
        when(callMock.execute()).thenReturn(Response.success(ResponseBody.create(okhttp3.MediaType.parse("text/html"), new byte[0])));

        commonPreparation();
        when(electionInformationAdminClient.saveElectoralData(eq(ElectoralDataEIResource.ELECTORAL_DATA_PATH), 
            eq(TENANT_ID), eq(ELECTION_EVENT_ID), eq(ELECTORAL_AUTHORITY_ID), eq(ADMIN_BOARD_ID),
            eq(X_FORWARDER_VALUE), eq(TRACK_ID), any()))
                .thenReturn(callMock);

        int status = target(URL_SAVE_EE_CONTEXT_DATA).resolveTemplate("tenantId", TENANT_ID).resolveTemplate("electionEventId", ELECTION_EVENT_ID)
                .resolveTemplate("adminBoardId", ADMIN_BOARD_ID).resolveTemplate("electoralAuthorityId", ELECTORAL_AUTHORITY_ID)
                .request()
                .post(Entity.entity(new ArrayList<Object>(), MediaType.APPLICATION_JSON_TYPE)).getStatus();
        
        Assert.assertEquals(mockedInvocationStatus, status);
    }
    
    @Test
    public void saveElectionEventDataUnsuccessful() throws IOException {

        int mockedInvocationStatus = 500;

        @SuppressWarnings("unchecked")
		Call<ResponseBody> callMock = (Call<ResponseBody>) Mockito.mock(Call.class);
    	when(callMock.execute()).thenReturn(Response.error(mockedInvocationStatus,  ResponseBody.create(okhttp3.MediaType.parse("text/html"), new byte[0])));

        commonPreparation();
        when(electionInformationAdminClient.saveElectoralData(eq(ElectoralDataEIResource.ELECTORAL_DATA_PATH), 
            eq(TENANT_ID), eq(ELECTION_EVENT_ID), eq(ELECTORAL_AUTHORITY_ID), eq(ADMIN_BOARD_ID),
            eq(X_FORWARDER_VALUE), eq(TRACK_ID), any()))
                .thenReturn(callMock);

        int status = target(URL_SAVE_EE_CONTEXT_DATA).resolveTemplate("tenantId", TENANT_ID).resolveTemplate("electionEventId", ELECTION_EVENT_ID)
                .resolveTemplate("adminBoardId", ADMIN_BOARD_ID).resolveTemplate("electoralAuthorityId", ELECTORAL_AUTHORITY_ID)
                .request()
                .post(Entity.entity(new ArrayList<Object>(), MediaType.APPLICATION_JSON_TYPE)).getStatus();
        
        Assert.assertEquals(mockedInvocationStatus, status);
    }
    

    private void commonPreparation() {

        Deencapsulation.setField(sut, "trackIdGenerator", trackIdGenerator);

        when(servletRequest.getHeader(eq(RestClientInterceptor.HEADER_ORIGINATOR))).thenReturn(ORIGINATOR);
        when(servletRequest.getHeader(eq(RestClientInterceptor.HEADER_SIGNATURE))).thenReturn(SIGNATURE);
        when(servletRequest.getRemoteAddr()).thenReturn("");
        when(servletRequest.getLocalAddr()).thenReturn("");
        when(trackIdGenerator.generate()).thenReturn(TRACK_ID);
    }

    @Override
    protected Application configure() {

        System.setProperty("ELECTION_INFORMATION_CONTEXT_URL", "localhost");
        //init the mocks before instantiating the SUT. We cannot have this and MockitoJrunner, otherwise the mocks will
        //not be the ones that are used in the SUT constructor
        MockitoAnnotations.initMocks(this);

        AbstractBinder binder = new AbstractBinder() {
            @Override
            protected void configure() {
            	bind(logger).to(Logger.class);
                bind(servletRequest).to(HttpServletRequest.class);
            }
        };
        sut = new ElectoralDataEIResource(electionInformationAdminClient, trackIdGenerator);
        forceSet(TestProperties.CONTAINER_PORT, "0");
        return new ResourceConfig().register(sut).register(binder);
    }
}
